const AWS = require('aws-sdk')
const zlib = require('zlib');
const CW = new AWS.CloudWatch()
const APIGW = new AWS.APIGateway()

exports.handler = async function(event,context){
    return new Promise((resolve,reject) => {

        console.log("Event: "+JSON.stringify(event))
        var payload = Buffer.from(event.awslogs.data, 'base64');
        zlib.gunzip(payload, function(e, result) {
            if (e) { 
                console.error(JSON.stringify(e, null, 2))
                reject(e)
            } else {
                //début du traitement du log stream fourni
                result = JSON.parse(result.toString('ascii'));
                console.log("Event Data after decoding: ", JSON.stringify(result, null, 2));
                
                var promises = []

                result.logEvents.forEach(log => {
                    //for each log, get api name and push a metric data object associated in the payload for PutMetricData
                    var message = JSON.parse(log.message)
                    var getRestApiPromise = APIGW.getRestApi({restApiId:message.apiId}).promise()
                    promises.push(new Promise((res,rej)=>{
                        getRestApiPromise.then(data=>{
                            console.log("getRestApi result: "+JSON.stringify(data))
                            res(
                                {
                                    MetricName: '400 Errors', 
                                    Dimensions: [
                                        {
                                            Name: 'ApiName',
                                            Value:  data.name
                                        },
                                        {
                                            Name: 'Stage',
                                            Value:  message.stage
                                        },
                                        {
                                            Name: 'Method',
                                            Value:  message.httpMethod
                                        },
                                        {
                                            Name: 'Resource',
                                            Value:  message.resourcePath
                                        },
                                    ],
                                    Timestamp: Math.floor(log.timestamp/1000),
                                    Unit: "Count",
                                    Value: message.status === "400" ? "1":"0",
                                }
                            )
                        }).catch(err=>{
                            rej(err)
                        })
                    }))
                });
                
                //when all logs are processed, send the PutMetricData request
                Promise.all(promises).then(metrics => { //if all the promises (cf line 26) succeed (equivalent to all the getRestApiPromise)
                    const params = {
                        MetricData:metrics,
                        Namespace: "APIGW/custom"
                    };
                    console.log("PutMetricData parameters : "+JSON.stringify(params))
                    CW.putMetricData(params).promise()
                        .then(data=>{
                            console.log("PutMetricData succeeded :"+JSON.stringify(data))
                            resolve(data)
                        })
                        .catch(err=>{
                            console.error(err)
                            reject(err)
                        });
                }).catch( err => { //if one of the promise (cf line 26) fails
                    console.error(err)
                    reject(err)
                });
            }
        });
    })
};